@@ -2,6 +2,7 @@ from __future__ import annotations
 
 from collections import namedtuple
 from datetime import date, datetime, timezone
+from decimal import Decimal
 from enum import Enum
 from typing import List, Sequence, Union
 
@@ -206,7 +207,7 @@ def format_value(value: ParameterType) -> str:
     """For python value to be used in a SQL query"""
     if isinstance(value, bool):
         return str(int(value))
-    if isinstance(value, (int, float)):
+    if isinstance(value, (int, float, Decimal)):
         return str(value)
     elif isinstance(value, str):
         return f"'{''.join(escape_chars.get(c, c) for c in value)}'"
