@@ -1,5 +1,5 @@
 from logging import getLogger
-from typing import List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
 from firebolt.common.exception import FireboltError
 from firebolt.common.urls import (
@@ -9,8 +9,7 @@ from firebolt.common.urls import (
     ENGINES_BY_IDS_URL,
 )
 from firebolt.common.util import prune_dict
-from firebolt.model import FireboltBaseModel
-from firebolt.model.engine import Engine, EngineSettings
+from firebolt.model.engine import Engine, EngineSettings, _EngineCreateRequest
 from firebolt.model.engine_revision import (
     EngineRevision,
     EngineRevisionSpecification,
@@ -60,11 +59,11 @@ class EngineService(BaseService):
 
     def get_many(
         self,
-        name_contains: str,
-        current_status_eq: str,
-        current_status_not_eq: str,
-        region_eq: str,
-        order_by: Union[str, EngineOrder],
+        name_contains: Optional[str] = None,
+        current_status_eq: Optional[str] = None,
+        current_status_not_eq: Optional[str] = None,
+        region_eq: Optional[str] = None,
+        order_by: Optional[Union[str, EngineOrder]] = None,
     ) -> List[Engine]:
         """
         Get a list of engines on Firebolt.
@@ -81,7 +80,13 @@ class EngineService(BaseService):
         """
 
         if isinstance(order_by, str):
-            order_by = EngineOrder[order_by]
+            order_by = EngineOrder[order_by].name
+
+        if region_eq is not None:
+            region_eq = self.resource_manager.regions.get_by_name(
+                name=region_eq
+            ).key.region_id
+
         response = self.client.get(
             url=ACCOUNT_ENGINES_URL.format(account_id=self.account_id),
             params=prune_dict(
@@ -90,16 +95,14 @@ class EngineService(BaseService):
                     "filter.name_contains": name_contains,
                     "filter.current_status_eq": current_status_eq,
                     "filter.current_status_not_eq": current_status_not_eq,
-                    "filter.compute_region_id_region_id_eq": self.resource_manager.regions.get_by_name(  # noqa: E501
-                        name=region_eq
-                    ),
-                    "order_by": order_by.name,
+                    "filter.compute_region_id_region_id_eq": region_eq,
+                    "order_by": order_by,
                 }
             ),
         )
         return [
-            Engine.parse_obj_with_service(obj=e, engine_service=self)
-            for e in response.json()["engines"]
+            Engine.parse_obj_with_service(obj=e["node"], engine_service=self)
+            for e in response.json()["edges"]
         ]
 
     def create(
@@ -112,6 +115,8 @@ class EngineService(BaseService):
         auto_stop: int = 20,
         warmup: Union[str, WarmupMethod] = WarmupMethod.PRELOAD_INDEXES,
         description: str = "",
+        engine_settings_kwargs: Dict[str, Any] = {},
+        revision_spec_kwargs: Dict[str, Any] = {},
     ) -> Engine:
         """
         Create a new Engine.
@@ -161,6 +166,7 @@ class EngineService(BaseService):
                 engine_type=engine_type,
                 auto_stop_delay_duration=f"{auto_stop * 60}s",
                 warm_up=warmup,
+                **engine_settings_kwargs,
             ),
         )
 
@@ -182,11 +188,8 @@ class EngineService(BaseService):
             specification=EngineRevisionSpecification(
                 db_compute_instances_type_key=instance_type_key,
                 db_compute_instances_count=scale,
-                db_compute_instances_use_spot=False,
-                db_version="",
                 proxy_instances_type_key=instance_type_key,
-                proxy_instances_count=1,
-                proxy_version="",
+                **revision_spec_kwargs,
             )
         )
 
@@ -206,13 +209,6 @@ class EngineService(BaseService):
             The newly created engine.
         """
 
-        class _EngineCreateRequest(FireboltBaseModel):
-            """Helper model for sending Engine create requests."""
-
-            account_id: str
-            engine: Engine
-            engine_revision: Optional[EngineRevision]
-
         response = self.client.post(
             url=ACCOUNT_ENGINES_URL.format(account_id=self.account_id),
             headers={"Content-type": "application/json"},
